@@ -3,6 +3,15 @@ freeze_eval <- names(.GlobalEnv)
 
 # defCondition ----
 
+# defData ----
+test_that("defData throws errors", {
+  expect_error(defData(dist = "nonrandom", formula = 7, id = "idnum"), class = "simstudy::missingArgument")
+  expect_error(defData(varname = "xnr", dist = "nonrandom", id = "idnum"), class = "simstudy::missingArgument")
+})
+
+# defRead ----
+
+
 # .evalDef ----
 test_that("checks combine in .evalDef correctly", {
   skip_on_cran()
@@ -155,7 +164,7 @@ test_that("defRepeat works.", {
   expect_silent(
     defRepeat(nVars = 4, prefix = "g", formula = "1/3;1/3;1/3", variance = 0, dist = "categorical")
   )
-  
+
   def <- defData(varname = "a", formula = "1;1", dist = "trtAssign")
   expect_silent(
     defRepeat(def, 8, "b", formula = "5 + a", variance = 3, dist = "normal")
@@ -164,11 +173,14 @@ test_that("defRepeat works.", {
 
 test_that("defRepeat throws errors correctly.", {
   expect_error(defRepeat(prefix = "b", formula = 5, variance = 3, dist = "normal"),
-    class = "simstudy::missingArgument")
-  expect_error(defRepeat(nVars = 8, formula = 5, variance = 3, dist = "normal"), 
-    class = "simstudy::missingArgument")
+    class = "simstudy::missingArgument"
+  )
+  expect_error(defRepeat(nVars = 8, formula = 5, variance = 3, dist = "normal"),
+    class = "simstudy::missingArgument"
+  )
   expect_error(defRepeat(nVars = 8, prefix = "b", variance = 3, dist = "normal"),
-    class = "simstudy::missingArgument")
+    class = "simstudy::missingArgument"
+  )
   expect_error(defRepeat(nVars = 4, prefix = "b", formula = "5 + a", variance = 3, dist = "normal"))
 })
 
